
from setuptools import setup

setup(name='zwatershed',
      version='0.1',
      description='Fast watersheds',
      url='https://github.com/TuragaLab/zwatershed',
      author='Chandan Singh',
      author_email='cs3hq@virginia.edu',
      license='MIT',
      packages=['zwatershed'],
      zip_safe=False)
